@@ -30,7 +30,6 @@
 
 #include "internal.h"
 #ifdef _WIN32
-#include <windows.h>
 #include <bcrypt.h>
 #include <strsafe.h>
 #else
@@ -39,6 +38,7 @@
 #include <openssl/des.h>
 #include <openssl/evp.h>
 #include <openssl/rand.h>
+#include <openssl/sha.h>
 #endif
 
 #include <stddef.h>
@@ -46,28 +46,14 @@
 #include <stdlib.h>
 #include <string.h>
 
-/*
-** Definitions
-*/
+ /*
+ ** Definitions
+ */
 
-/* crypt defines */
+ /* crypt defines */
 
 #ifdef _WIN32
-
 #define strcasecmp _stricmp
-
-struct _cipher_key {
-    BCRYPT_KEY_HANDLE hKey;
-};
-
-#else
-
-struct _cipher_key {
-  const EVP_CIPHER *cipher;
-  EVP_CIPHER_CTX *ctx;
-  unsigned char key[EVP_MAX_KEY_LENGTH];
-};
-
 #endif
 
 /* config defines */
@@ -80,189 +66,14 @@ struct _cipher_key {
 
 #define _ENV_PREFIX    "YUBIKEY_PIV_"
 
-char *_strip_ws(char *sz);
-setting_bool_t _get_bool_config(const char *sz_setting);
-setting_bool_t _get_bool_env(const char *sz_setting);
+char* _strip_ws(char* sz);
+setting_bool_t _get_bool_config(const char* sz_setting);
+setting_bool_t _get_bool_env(const char* sz_setting);
 
 /*
 ** Methods
 */
 
-#ifdef _WIN32
-
-static BCRYPT_ALG_HANDLE bcrypt_algo(unsigned char algo) {
-	switch (algo) {
-	case YKPIV_ALGO_3DES:
-    return BCRYPT_3DES_ECB_ALG_HANDLE;
-	case YKPIV_ALGO_AES128:
-  case YKPIV_ALGO_AES192:
-  case YKPIV_ALGO_AES256:
-    return BCRYPT_AES_ECB_ALG_HANDLE;
-  default:
-    return NULL;
-	}
-}
-
-cipher_rc cipher_import_key(unsigned char algo, const unsigned char *keyraw, uint32_t keyrawlen, cipher_key *key) {
-  *key = calloc(1, sizeof(**key));
-	if (!*key) {
-    return CIPHER_MEMORY_ERROR;
-  }
-	if(!BCRYPT_SUCCESS(BCryptGenerateSymmetricKey(bcrypt_algo(algo), &(*key)->hKey, NULL, 0, (PUCHAR)keyraw, keyrawlen, 0))) {
-		return CIPHER_INVALID_PARAMETER;
-	}
-	return 0;
-}
-
-cipher_rc cipher_destroy_key(cipher_key key) {
-	if (key == NULL) {
-    return CIPHER_MEMORY_ERROR;
-  }
-	if(!BCRYPT_SUCCESS(BCryptDestroyKey(key->hKey))) {
-    return CIPHER_INVALID_PARAMETER;
-  }
-	free(key);
-	return CIPHER_OK;
-}
-
-cipher_rc cipher_encrypt(cipher_key key, const unsigned char* in, uint32_t inlen, unsigned char* out, uint32_t* outlen) {
-	if (key == NULL) {
-    return CIPHER_MEMORY_ERROR;
-  }
-	if(!BCRYPT_SUCCESS(BCryptEncrypt(key->hKey, (PUCHAR)in, inlen, NULL, NULL, 0, out, *outlen, (PULONG)outlen, 0))) {
-    return CIPHER_INVALID_PARAMETER;
-  }
-  return CIPHER_OK;
-}
-
-cipher_rc cipher_decrypt(cipher_key key, const unsigned char* in, uint32_t inlen, unsigned char* out, uint32_t* outlen) {
-	if (key == NULL) {
-    return CIPHER_MEMORY_ERROR;
-  }
-	if(!BCRYPT_SUCCESS(BCryptDecrypt(key->hKey, (PUCHAR)in, inlen, NULL, NULL, 0, out, *outlen, (PULONG)outlen, 0))) {
-    return CIPHER_INVALID_PARAMETER;
-  }
-  return CIPHER_OK;
-}
-
-uint32_t cipher_blocksize(cipher_key key) {
-	if (key == NULL) {
-    return 0;
-  }
-  DWORD size = 0;
-  ULONG len = 0;
-	if(!BCRYPT_SUCCESS(BCryptGetProperty(key->hKey, BCRYPT_BLOCK_LENGTH, (PUCHAR)&size, sizeof(size), &len, 0))) {
-    return 0;
-  }
-  return size;
-}
-
-#else
-
-static int encrypt_ex(const uint8_t *in, uint8_t *out, int len,
-                      const uint8_t *iv, int enc, cipher_key ctx) {
-  if (EVP_CipherInit_ex(ctx->ctx, ctx->cipher, NULL, ctx->key, iv, enc) != 1) {
-    return -1;
-  }
-  if (EVP_CIPHER_CTX_set_padding(ctx->ctx, 0) != 1) {
-    return -2;
-  }
-  int update_len = len;
-  if (EVP_CipherUpdate(ctx->ctx, out, &update_len, in, len) != 1) {
-    return -3;
-  }
-  int final_len = len - update_len;
-  if (EVP_CipherFinal_ex(ctx->ctx, out + update_len, &final_len) != 1) {
-    return -4;
-  }
-  if (update_len + final_len != len) {
-    return -5;
-  }
-  return 0;
-}
-
-cipher_rc cipher_import_key(unsigned char algo, const unsigned char* keyraw, uint32_t keyrawlen, cipher_key* key) {
-  cipher_rc rc = CIPHER_OK;
-
-  *key = calloc(1, sizeof(**key));
-  (*key)->ctx = EVP_CIPHER_CTX_new();
-
-  switch (algo) {
-  case YKPIV_ALGO_3DES:
-    (*key)->cipher = EVP_des_ede3_ecb();
-    break;
-  case YKPIV_ALGO_AES128:
-    (*key)->cipher = EVP_aes_128_ecb();
-    break;
-  case YKPIV_ALGO_AES192:
-    (*key)->cipher = EVP_aes_192_ecb();
-    break;
-  case YKPIV_ALGO_AES256:
-    (*key)->cipher = EVP_aes_256_ecb();
-    break;
-  default:
-    rc = CIPHER_INVALID_PARAMETER;
-    goto ERROR_EXIT;
-  }
-
-  if((*key)->cipher == NULL || EVP_CIPHER_key_length((*key)->cipher) != keyrawlen) {
-    rc = CIPHER_INVALID_PARAMETER;
-    goto ERROR_EXIT;
-  }
-
-  memcpy((*key)->key, keyraw, keyrawlen);
-
-EXIT:
-  return rc;
-
-ERROR_EXIT:
-  if (key) {
-    cipher_destroy_key(*key);
-    *key = NULL;
-  }
-  goto EXIT;
-}
-
-cipher_rc cipher_destroy_key(cipher_key key) {
-  if (key) {
-    EVP_CIPHER_CTX_free(key->ctx);
-    memset(key, 0, sizeof(*key));
-    free(key);
-  }
-  return CIPHER_OK;
-}
-
-cipher_rc cipher_encrypt(cipher_key key, const unsigned char* in, uint32_t inlen, unsigned char* out, uint32_t* outlen) {
-  cipher_rc rc = CIPHER_OK;
-
-  if (!key || !outlen || (*outlen < inlen) || !in || !out) { rc = CIPHER_INVALID_PARAMETER; goto EXIT; }
-
-  rc = encrypt_ex(in, out, inlen, NULL, 1, key);
-
-EXIT:
-  return rc;
-}
-
-cipher_rc cipher_decrypt(cipher_key key, const unsigned char* in, uint32_t inlen, unsigned char* out, uint32_t* outlen) {
-  cipher_rc rc = CIPHER_OK;
-
-  if (!key || !outlen || (*outlen < inlen) || !in || !out) { rc = CIPHER_INVALID_PARAMETER; goto EXIT; }
-
-  rc = encrypt_ex(in, out, inlen, NULL, 0, key);
-
-EXIT:
-  return rc;
-}
-
-uint32_t cipher_blocksize(cipher_key key) {
-  if(key) {
-    return EVP_CIPHER_block_size(key->cipher);
-  }
-  return 0;
-}
-
-#endif
-
 bool yk_des_is_weak_key(const unsigned char *key, const size_t cb_key) {
 #ifdef _WIN32
   bool rv = false;
@@ -331,7 +142,15 @@ prng_rc _ykpiv_prng_generate(unsigned char *buffer, const size_t cb_req) {
   prng_rc rc = PRNG_OK;
 
 #ifdef _WIN32
-  if (!BCRYPT_SUCCESS(BCryptGenRandom(BCRYPT_RNG_ALG_HANDLE, buffer, (ULONG)cb_req, 0))) {
+  BCRYPT_ALG_HANDLE hAlg = 0;
+
+  if (BCRYPT_SUCCESS(BCryptOpenAlgorithmProvider(&hAlg, BCRYPT_RNG_ALGORITHM, NULL, 0))) {
+    if (!BCRYPT_SUCCESS(BCryptGenRandom(hAlg, buffer, (ULONG)cb_req, 0))) {
+      rc = PRNG_GENERAL_ERROR;
+    }
+    BCryptCloseAlgorithmProvider(hAlg, 0);
+  }
+  else {
     rc = PRNG_GENERAL_ERROR;
   }
 #else
@@ -358,8 +177,16 @@ pkcs5_rc pkcs5_pbkdf2_sha1(const uint8_t* password, const size_t cb_password, co
 
     /* suppress const qualifier warning b/c BCrypt doesn't take const input buffers */
 #pragma warning(suppress: 4090)
-  if (!BCRYPT_SUCCESS(BCryptDeriveKeyPBKDF2(BCRYPT_HMAC_SHA1_ALG_HANDLE, (PUCHAR)password, (ULONG)cb_password, (PUCHAR)salt, (ULONG)cb_salt, iterations, key, (ULONG)cb_key, 0)))
-  {
+  BCRYPT_ALG_HANDLE hPrf = NULL;
+
+  if (BCRYPT_SUCCESS(BCryptOpenAlgorithmProvider(&hPrf, BCRYPT_SHA1_ALGORITHM, NULL, BCRYPT_ALG_HANDLE_HMAC_FLAG))) {
+    if (!BCRYPT_SUCCESS(BCryptDeriveKeyPBKDF2(hPrf, (PUCHAR)password, (ULONG)cb_password, (PUCHAR)salt, (ULONG)cb_salt, iterations, (PUCHAR)key, (ULONG)cb_key, 0)))
+    {
+      rc = PKCS5_GENERAL_ERROR;
+    }
+    BCryptCloseAlgorithmProvider(hPrf, 0);
+  }
+  else {
     rc = PKCS5_GENERAL_ERROR;
   }
 
@@ -373,6 +200,39 @@ pkcs5_rc pkcs5_pbkdf2_sha1(const uint8_t* password, const size_t cb_password, co
   return rc;
 }
 
+unsigned char* hash_sha256(const unsigned char* data, size_t count, unsigned char* md_buf) {
+  unsigned char* rc = 0;
+#ifdef _WIN32
+  BCRYPT_ALG_HANDLE hAlg = NULL;
+  BCRYPT_HASH_HANDLE hHash = NULL;
+
+  if (!BCRYPT_SUCCESS(BCryptOpenAlgorithmProvider(&hAlg, BCRYPT_SHA256_ALGORITHM, NULL, 0))) {
+    goto cleanup;
+  }
+
+  if (!BCRYPT_SUCCESS(BCryptCreateHash(hAlg, &hHash, NULL, 0, NULL, 0, 0))) {
+    goto cleanup;
+  }
+
+  if (!BCRYPT_SUCCESS(BCryptHashData(hHash, (PUCHAR)data, (ULONG)count, 0))) {
+    goto cleanup;
+  }
+
+  if (!BCRYPT_SUCCESS(BCryptFinishHash(hHash, md_buf, 32, 0))) {
+    goto cleanup;
+  }
+
+  rc = md_buf;
+
+cleanup:
+  if (hAlg) BCryptCloseAlgorithmProvider(hAlg, 0);
+
+#else
+  rc = SHA256(data, count, md_buf);
+#endif
+  return rc;
+}
+
 /* settings */
 
 char *_strip_ws(char *sz) {
