@@ -31,12 +31,13 @@
 #include <string.h>
 #include <stdio.h>
 #include <time.h>
-
+ 
+#ifdef USE_CERT_COMPRESS
 #include <zlib.h>
+#endif
 
 #include "internal.h"
 #include "ykpiv.h"
-#include "util.h"
 
 #define MAX(a,b) (a) > (b) ? (a) : (b)
 #define MIN(a,b) (a) < (b) ? (a) : (b)
@@ -90,6 +91,18 @@ static size_t _obj_size_max(ykpiv_state *state) {
   return (state && state->model == DEVTYPE_NEOr3) ? CB_OBJ_MAX_NEO : CB_OBJ_MAX;
 }
 
+static unsigned long get_length_size(unsigned long length) {
+  if (length < 0x80) {
+    return 1;
+  }
+  else if (length < 0x100) {
+    return 2;
+  }
+  else {
+    return 3;
+  }
+}
+
 /*
 ** YKPIV Utility API - aggregate functions and slightly nicer interface
 */
@@ -103,9 +116,10 @@ ykpiv_rc ykpiv_util_get_cardid(ykpiv_state *state, ykpiv_cardid *cardid) {
   uint8_t tag = 0;
 
   if (!cardid) return YKPIV_ARGUMENT_ERROR;
+   uint8_t scp11 = state->scp11_state.security_level;
 
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   if ((res = _ykpiv_fetch_object(state, YKPIV_OBJ_CHUID, buf, &len)) == YKPIV_OK) {
     p_temp = buf;
@@ -153,6 +167,7 @@ ykpiv_rc ykpiv_util_set_cardid(ykpiv_state *state, const ykpiv_cardid *cardid) {
   size_t len = 0;
 
   if (!state) return YKPIV_ARGUMENT_ERROR;
+  uint8_t scp11 = state->scp11_state.security_level;
 
   if (!cardid) {
     if (PRNG_OK != _ykpiv_prng_generate(id, sizeof(id))) {
@@ -164,7 +179,7 @@ ykpiv_rc ykpiv_util_set_cardid(ykpiv_state *state, const ykpiv_cardid *cardid) {
   }
 
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   memcpy(buf, CHUID_TMPL, sizeof(CHUID_TMPL));
   memcpy(buf + CHUID_GUID_OFFS, id, sizeof(id));
@@ -185,8 +200,9 @@ ykpiv_rc ykpiv_util_get_cccid(ykpiv_state *state, ykpiv_cccid *ccc) {
 
   if (!ccc) return YKPIV_ARGUMENT_ERROR;
 
+  uint8_t scp11 = state->scp11_state.security_level;
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   res = _ykpiv_fetch_object(state, YKPIV_OBJ_CAPABILITY, buf, &len);
   if (YKPIV_OK == res) {
@@ -211,6 +227,7 @@ ykpiv_rc ykpiv_util_set_cccid(ykpiv_state *state, const ykpiv_cccid *ccc) {
   size_t len = 0;
 
   if (!state) return YKPIV_ARGUMENT_ERROR;
+  uint8_t scp11 = state->scp11_state.security_level;
 
   if (!ccc) {
     if (PRNG_OK != _ykpiv_prng_generate(id, sizeof(id))) {
@@ -222,7 +239,7 @@ ykpiv_rc ykpiv_util_set_cccid(ykpiv_state *state, const ykpiv_cccid *ccc) {
   }
 
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   len = sizeof(CCC_TMPL);
   memcpy(buf, CCC_TMPL, len);
@@ -284,8 +301,9 @@ ykpiv_rc ykpiv_util_list_keys(ykpiv_state *state, uint8_t *key_count, ykpiv_key
 
   if ((NULL == data) || (NULL == data_len) || (NULL == key_count)) { return YKPIV_ARGUMENT_ERROR; }
 
+  uint8_t scp11 = state->scp11_state.security_level;
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   // init return parameters
   *key_count = 0;
@@ -315,6 +333,7 @@ ykpiv_rc ykpiv_util_list_keys(ykpiv_state *state, uint8_t *key_count, ykpiv_key
           res = YKPIV_MEMORY_ERROR;
           goto Cleanup;
         }
+        yc_memzero(pTemp + cbData, cbRealloc); // clear newly allocated memory
         pData = pTemp;
         pTemp = NULL;
       }
@@ -369,8 +388,9 @@ ykpiv_rc ykpiv_util_read_cert(ykpiv_state *state, uint8_t slot, uint8_t **data,
 
   if ((NULL == data )|| (NULL == data_len)) return YKPIV_ARGUMENT_ERROR;
 
+  uint8_t scp11 = state->scp11_state.security_level;
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   *data = 0;
   *data_len = 0;
@@ -402,9 +422,9 @@ Cleanup:
 
 ykpiv_rc ykpiv_util_write_cert(ykpiv_state *state, uint8_t slot, uint8_t *data, size_t data_len, uint8_t certinfo) {
   ykpiv_rc res = YKPIV_OK;
-
+  uint8_t scp11 = state->scp11_state.security_level;
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   res = _write_certificate(state, slot, data, data_len, certinfo);
 
@@ -428,10 +448,11 @@ ykpiv_rc ykpiv_util_block_puk(ykpiv_state *state) {
   size_t  cb_item = 0;
   uint8_t flags = 0;
 
-  if (NULL == state) return YKPIV_ARGUMENT_ERROR;
+  if (!state) return YKPIV_ARGUMENT_ERROR;
+  uint8_t scp11 = state->scp11_state.security_level;
 
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   while (tries != 0) {
     if (YKPIV_OK == (res = ykpiv_change_puk(state, (const char*)puk, sizeof(puk), (const char*)puk, sizeof(puk), &tries))) {
@@ -486,8 +507,10 @@ ykpiv_rc ykpiv_util_read_mscmap(ykpiv_state *state, ykpiv_container **containers
   uint8_t *ptr = NULL;
 
   if ((NULL == containers) || (NULL == n_containers)) { res = YKPIV_ARGUMENT_ERROR; goto Cleanup; }
+
+  uint8_t scp11 = state->scp11_state.security_level;
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   *containers = 0;
   *n_containers = 0;
@@ -533,9 +556,10 @@ ykpiv_rc ykpiv_util_write_mscmap(ykpiv_state *state, ykpiv_container *containers
   size_t offset = 0;
   size_t req_len = 0;
   size_t data_len = n_containers * sizeof(ykpiv_container);
+  uint8_t scp11 = state->scp11_state.security_level;
 
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   // check if data and data_len are zero, this means that
   // we intend to delete the object
@@ -593,8 +617,9 @@ ykpiv_rc ykpiv_util_read_msroots(ykpiv_state *state, uint8_t **data, size_t *dat
 
   if (!data || !data_len) return YKPIV_ARGUMENT_ERROR;
 
+  uint8_t scp11 = state->scp11_state.security_level;
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   *data = 0;
   *data_len = 0;
@@ -679,9 +704,10 @@ ykpiv_rc ykpiv_util_write_msroots(ykpiv_state *state, uint8_t *data, size_t data
   size_t n_objs = 0;
   unsigned int i = 0;
   size_t cb_obj_max = _obj_size_max(state);
+  uint8_t scp11 = state->scp11_state.security_level;
 
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   // check if either data and data_len are zero, this means that
   // we intend to delete the object
@@ -766,6 +792,7 @@ ykpiv_rc ykpiv_util_generate_key(ykpiv_state *state, uint8_t slot, uint8_t algor
     "The default behavior will change in a future Yubico release.";
 
   if (!state) return YKPIV_ARGUMENT_ERROR;
+  uint8_t scp11 = state->scp11_state.security_level;
 
   if ((algorithm == YKPIV_ALGO_RSA3072 || algorithm == YKPIV_ALGO_RSA4096 || YKPIV_IS_25519(algorithm))
        && !is_version_compatible(state, 5, 7, 0)) {
@@ -833,7 +860,7 @@ ykpiv_rc ykpiv_util_generate_key(ykpiv_state *state, uint8_t slot, uint8_t algor
   }
 
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   templ[3] = slot;
 
@@ -866,7 +893,7 @@ ykpiv_rc ykpiv_util_generate_key(ykpiv_state *state, uint8_t slot, uint8_t algor
   if (YKPIV_OK != (res = _ykpiv_transfer_data(state, templ, in_data, (unsigned long)(in_ptr - in_data), data, &recv_len, &sw))) {
     goto Cleanup;
   }
-  res = ykpiv_translate_sw(sw);
+  res = ykpiv_translate_sw_ex(__FUNCTION__, sw);
   if (res != YKPIV_OK) {
     DBG("Failed to generate new key");
     goto Cleanup;
@@ -1000,6 +1027,8 @@ ykpiv_rc ykpiv_util_get_config(ykpiv_state *state, ykpiv_config *config) {
   if (NULL == state) return YKPIV_ARGUMENT_ERROR;
   if (NULL == config) return YKPIV_ARGUMENT_ERROR;
 
+  uint8_t scp11 = state->scp11_state.security_level;
+
   // initialize default values
 
   config->puk_blocked = false;
@@ -1008,7 +1037,7 @@ ykpiv_rc ykpiv_util_get_config(ykpiv_state *state, ykpiv_config *config) {
   config->mgm_type = YKPIV_CONFIG_MGM_MANUAL;
 
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   /* recover admin data */
   if (YKPIV_OK == _read_metadata(state, TAG_ADMIN, data, &cb_data)) {
@@ -1081,9 +1110,10 @@ ykpiv_rc ykpiv_util_set_pin_last_changed(ykpiv_state *state) {
   time_t   tnow = 0;
 
   if (NULL == state) return YKPIV_ARGUMENT_ERROR;
+  uint8_t scp11 = state->scp11_state.security_level;
 
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   /* recover admin data */
   if (YKPIV_OK != (ykrc = _read_metadata(state, TAG_ADMIN, data, &cb_data))) {
@@ -1119,8 +1149,9 @@ ykpiv_rc ykpiv_util_get_derived_mgm(ykpiv_state *state, const uint8_t *pin, cons
   if (NULL == state) return YKPIV_ARGUMENT_ERROR;
   if ((NULL == pin) || (0 == pin_len) || (NULL == mgm)) return YKPIV_ARGUMENT_ERROR;
 
+  uint8_t scp11 = state->scp11_state.security_level;
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   /* recover management key */
   if (YKPIV_OK == (res = _read_metadata(state, TAG_ADMIN, data, &cb_data))) {
@@ -1154,9 +1185,10 @@ ykpiv_rc ykpiv_util_get_protected_mgm(ykpiv_state *state, ykpiv_mgm *mgm) {
 
   if (NULL == state) return YKPIV_ARGUMENT_ERROR;
   if (NULL == mgm) return YKPIV_ARGUMENT_ERROR;
+  uint8_t scp11 = state->scp11_state.security_level;
 
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) return res;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   if (YKPIV_OK != (res = _read_metadata(state, TAG_PROTECTED, data, &cb_data))) {
     DBG("could not read protected data, err = %d", res);
@@ -1190,9 +1222,10 @@ ykpiv_rc ykpiv_util_update_protected_mgm(ykpiv_state *state, ykpiv_mgm *mgm) {
   ykpiv_rc res = YKPIV_OK;
   uint8_t data[CB_BUF_MAX] = {0};
   size_t cb_data = sizeof(data);
+  uint8_t scp11 = state->scp11_state.security_level;
 
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) goto Cleanup;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   if (YKPIV_OK != (res = _read_metadata(state, TAG_PROTECTED, data, &cb_data))) {
     cb_data = 0; /* set current metadata blob size to zero, we'll add to the blank blob */
@@ -1230,6 +1263,7 @@ ykpiv_rc ykpiv_util_set_protected_mgm(ykpiv_state *state, ykpiv_mgm *mgm) {
   uint8_t  flags_1 = 0;
 
   if (NULL == state) return YKPIV_ARGUMENT_ERROR;
+   uint8_t scp11 = state->scp11_state.security_level;
 
   fGenerate = true;
   if (mgm) {
@@ -1245,7 +1279,7 @@ ykpiv_rc ykpiv_util_set_protected_mgm(ykpiv_state *state, ykpiv_mgm *mgm) {
   }
 
   if (YKPIV_OK != (res = _ykpiv_begin_transaction(state))) goto Cleanup;
-  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state))) goto Cleanup;
+  if (YKPIV_OK != (res = _ykpiv_ensure_application_selected(state, scp11))) goto Cleanup;
 
   /* try to set the mgm key as long as we don't encounter a fatal error */
   do {
@@ -1358,7 +1392,7 @@ ykpiv_rc ykpiv_util_reset(ykpiv_state *state) {
   if(res != YKPIV_OK) {
     return res;
   }
-  return ykpiv_translate_sw(sw);
+  return ykpiv_translate_sw_ex(__FUNCTION__, sw);
 }
 
 uint32_t ykpiv_util_slot_object(uint8_t slot) {
@@ -1499,7 +1533,7 @@ invalid_tlv:
   size_t offset = 0;
   size_t buf_len = 0;
 
-  unsigned long len_bytes = get_length_size(rawdata_len);
+  unsigned long len_bytes = get_length_size((unsigned long)rawdata_len);
 
    // calculate the required length of the encoded object
    buf_len = 1 /* cert tag */ + 3 /* compression tag + data*/ + 2 /* lrc */;
@@ -1531,7 +1565,7 @@ invalid_tlv:
 
   if (-1 == object_id) return YKPIV_INVALID_OBJECT;
 
-   unsigned char data[YKPIV_OBJ_MAX_SIZE * 10] = {0};
+   unsigned char data[YKPIV_OBJ_MAX_SIZE] = {0};
    unsigned long data_len = sizeof (data);
 
   if (YKPIV_OK == (res = _ykpiv_fetch_object(state, object_id, data, &data_len))) {
@@ -1740,7 +1774,7 @@ static ykpiv_rc _set_metadata_item(uint8_t *data, size_t *pcb_data, size_t cb_da
   cb_len = _ykpiv_get_length_size(cb_item);
 
   // length would cause buffer overflow, return error
-  if (*pcb_data + cb_len + cb_item > cb_data_max) {
+  if (*pcb_data + 1 + cb_len + cb_item > cb_data_max) {
     return YKPIV_GENERIC_ERROR;
   }
 
@@ -1845,4 +1879,4 @@ static ykpiv_rc _write_metadata(ykpiv_state *state, uint8_t tag, uint8_t *data,
   }
 
   return res;
-}
+}
\ No newline at end of file
