@@ -57,10 +57,13 @@
 #include <openssl/err.h>
 #include <openssl/evp.h>
 #include <openssl/crypto.h>
+#ifdef USE_CERT_COMPRESS
 #include <zlib.h>
+#endif
 
 #include "cmdline.h"
 #include "../common/util.h"
+#include "internal.h"
 
 #define MAX(a,b) (a) > (b) ? (a) : (b)
 
@@ -107,9 +110,9 @@ static void print_version(ykpiv_state *state, const char *output_file_name) {
   }
 
   if(ykpiv_get_version(state, version, sizeof(version)) == YKPIV_OK) {
-    fprintf(output_file, "Application version %s found.\n", version);
+    fprintf(output_file, "Firmware version %s found.\n", version);
   } else {
-    fprintf(stderr, "Failed to retrieve application version.\n");
+    fprintf(stderr, "Failed to retrieve firmware version.\n");
   }
 
   if(output_file != stdout) {
@@ -259,15 +262,54 @@ static EVP_PKEY* wrap_public_key(ykpiv_state *state, int algorithm, EVP_PKEY *pu
 }
 #endif
 
-static bool move_key(ykpiv_state *state, int from_slot, int to_slot) {
+static bool get_public_key(ykpiv_state *state, enum enum_slot slot, const char *output_file_name,
+                           enum enum_key_format key_format) {
+  int slot_hex = get_slot_hex(slot);
+  uint8_t data[YKPIV_OBJ_MAX_SIZE] = {0};
+  size_t data_len = sizeof(data);
+  FILE *output_file = NULL;
+  ykpiv_metadata md = {0};
+  EVP_PKEY *public_key = NULL;
+  ykpiv_rc rc = YKPIV_OK;
   bool ret = false;
-  ykpiv_rc res;
 
-  res = ykpiv_move_key(state, (uint8_t) (from_slot & 0xFF), (uint8_t) (to_slot & 0xFF));
-  if (res != YKPIV_OK) {
-    fprintf(stderr, "Failed to move key.\n");
-  } else {
+  output_file = open_file(output_file_name, key_file_mode(key_format, true));
+  if(!output_file) {
+    return false;
+  }
+
+  if (key_format != key_format_arg_PEM) {
+    fprintf(stderr, "Only PEM is supported as public_key output.\n");
+    goto pubkey_out;
+  }
+
+  if(ykpiv_get_metadata(state, slot_hex, data, &data_len) != YKPIV_OK) {
+    fprintf(stderr, "Metadata object for key was not found\n");
+    goto pubkey_out;
+  }
+  if((rc = ykpiv_util_parse_metadata(data, data_len, &md)) != YKPIV_OK) {
+    fprintf(stderr, "Failed to parse metadata: %s\n", ykpiv_strerror(rc));
+    goto pubkey_out;
+  }
+
+  if((rc = do_create_public_key(md.pubkey, md.pubkey_len, md.algorithm, &public_key)) != YKPIV_OK) {
+    fprintf(stderr, "Failed to create public key: %s\n", ykpiv_strerror(rc));
+    goto pubkey_out;
+  }
+
+  if(PEM_write_PUBKEY(output_file, public_key) == 1) {
     ret = true;
+  } else {
+    fprintf(stderr, "Failed to write public key in PEM format\n");
+    goto pubkey_out;
+  }
+
+pubkey_out:
+  if (output_file != stdout) {
+    fclose(output_file);
+  }
+  if (public_key) {
+    EVP_PKEY_free(public_key);
   }
   return ret;
 }
@@ -283,8 +325,6 @@ static bool generate_key(ykpiv_state *state, enum enum_slot slot,
   EVP_PKEY *public_key = NULL;
   RSA *rsa = NULL;
   EC_KEY *eckey = NULL;
-  EC_GROUP *group = NULL;
-  EC_POINT *ecpoint = NULL;
   uint8_t *mod = NULL;
   uint8_t *exp = NULL;
   uint8_t *point = NULL;
@@ -395,16 +435,19 @@ static bool generate_key(ykpiv_state *state, enum enum_slot slot,
     goto generate_out;
   }
 
+  // Check if there already is a certificate in the slot. If so, and since the private key in this same slot has just
+  // been generated, the certificate and the key inevitably do not match.
+  uint8_t *data = NULL;
+  size_t len = 0;
+  if (ykpiv_util_read_cert(state, key, &data, &len) == YKPIV_OK) {
+    fprintf(stderr, "\nBeware! The private key and the X509Certificate in slot %x do not match\n\n", key);
+    _ykpiv_free(state, data);
+  }
+
 generate_out:
   if (output_file != stdout) {
     fclose(output_file);
   }
-  if (group) {
-    EC_GROUP_clear_free(group);
-  }
-  if (ecpoint) {
-    EC_POINT_free(ecpoint);
-  }
   if (eckey) {
     EC_KEY_free(eckey);
   }
@@ -444,7 +487,7 @@ static bool reset(ykpiv_state *state, bool global) {
           fprintf(stderr,
                   "Global reset not supported on this YubiKey. Please refer to reset commands for specific applications instead\n");
         } else if(rc == YKPIV_ARGUMENT_ERROR) {
-          fprintf(stderr, "Reset failed, is 'scp11' flag used?\n");
+          fprintf(stderr, "Reset failed. If 'enc' flag is used, try to remove it\n");
         } else {
           fprintf(stderr, "Reset failed\n");
         }
@@ -649,9 +692,25 @@ static bool import_key(ykpiv_state *state, enum enum_key_format key_format,
     }
 #endif
 
-    ret = true;
-    if(rc != YKPIV_OK) {
-      ret = false;
+    if(rc == YKPIV_OK) {
+      ret = true;
+
+      // Notify user if existing key and certificate do not
+      uint8_t *certdata = NULL;
+      size_t certlen = 0;
+      if (ykpiv_util_read_cert(state, key, &certdata, &certlen) == YKPIV_OK) {
+        const unsigned char *certdata_ptr = certdata;
+        X509 *x509 = d2i_X509(NULL, &certdata_ptr, certlen);
+        if (x509 != NULL) {
+          EVP_PKEY *cert_key = X509_get_pubkey(x509);
+          if (EVP_PKEY_cmp(private_key, cert_key) != 1) {
+            fprintf(stderr, "\nBeware! The private key and the X509Certificate in slot %x do not match\n\n", key);
+          }
+          EVP_PKEY_free(cert_key);
+          X509_free(x509);
+        }
+        _ykpiv_free(state, certdata);
+      }
     }
   }
 
@@ -684,6 +743,7 @@ static bool import_cert(ykpiv_state *state, enum enum_key_format cert_format, in
   EVP_PKEY *private_key = NULL;
   int compress = YKPIV_CERTINFO_UNCOMPRESSED;
   int cert_len = -1;
+  int key = get_slot_hex(slot);
 
   input_file = open_file(input_file_name, key_file_mode(cert_format, false));
   if(!input_file) {
@@ -754,6 +814,7 @@ static bool import_cert(ykpiv_state *state, enum enum_key_format cert_format, in
         goto import_cert_out;
       }
     } else if(perform_compress) {
+#ifdef USE_CERT_COMPRESS
       unsigned char uncompressed_certdata[YKPIV_OBJ_MAX_SIZE*10] = {0};
       unsigned char *uncompressed_certptr = uncompressed_certdata;
       if(i2d_X509(cert, &uncompressed_certptr) < 0) {
@@ -784,6 +845,10 @@ static bool import_cert(ykpiv_state *state, enum enum_key_format cert_format, in
       }
       cert_len = zs.total_out;
       compress = YKPIV_CERTINFO_GZIP;
+#else
+      fprintf(stderr, "Certificate compression is not supported. Try to compress it manually and import it as GZIP.\n");
+      goto import_cert_out;
+#endif
     } else {
       if(i2d_X509(cert, &certptr) < 0) {
         fprintf(stderr, "Failed to encode X509 certificate\n");
@@ -791,13 +856,34 @@ static bool import_cert(ykpiv_state *state, enum enum_key_format cert_format, in
       }
     }
 
-    if ((res = ykpiv_util_write_cert(state, get_slot_hex(slot), certdata, (size_t)cert_len, compress)) != YKPIV_OK) {
+    if ((res = ykpiv_util_write_cert(state, key, certdata, (size_t) cert_len, compress)) != YKPIV_OK) {
       fprintf(stderr, "Failed commands with device: %s\n", ykpiv_strerror(res));
+      goto import_cert_out;
     } else {
       ret = true;
     }
   }
 
+  // Notify user if private key and certificate do not match
+  unsigned char metadata[YKPIV_OBJ_MAX_SIZE] = {0};
+  size_t metadata_len = sizeof(metadata);
+  ykpiv_metadata slot_md = {0};
+  if (ykpiv_get_metadata(state, key, metadata, &metadata_len) == YKPIV_OK &&
+      ykpiv_util_parse_metadata(metadata, metadata_len, &slot_md) == YKPIV_OK) {
+    EVP_PKEY *cert_pubkey = X509_get_pubkey(cert);
+    EVP_PKEY *md_pubkey = EVP_PKEY_new();
+    if (do_create_public_key(slot_md.pubkey, slot_md.pubkey_len, slot_md.algorithm, &md_pubkey) == YKPIV_OK &&
+        EVP_PKEY_cmp(cert_pubkey, md_pubkey) != 1) {
+        fprintf(stderr, "\nBeware! The private key and the X509Certificate in slot %x do not match\n\n", key);
+    }
+    if (cert_pubkey) {
+      EVP_PKEY_free(cert_pubkey);
+    }
+    if (md_pubkey) {
+      EVP_PKEY_free(md_pubkey);
+    }
+  }
+
 import_cert_out:
   if(cert) {
     X509_free(cert);
@@ -922,7 +1008,7 @@ static bool request_certificate(ykpiv_state *state, enum enum_key_format key_for
 
     if((rc = ykpiv_attest(state, key, buf, &buflen)) == YKPIV_OK) {
       STACK_OF(X509_EXTENSION) *exts = sk_X509_EXTENSION_new_null();
-      add_ext(exts, YKPIV_ATTESTATION_OID ".11", "ykpiv attestation", "Yubico PIV X.509 Attestation", buf, buflen);
+      add_ext(exts, YKPIV_ATTESTATION_OID ".1", "ykpiv attestation", "Yubico PIV Attestation", buf, buflen);
 
       unsigned char *pb = 0;
       size_t pblen = 0;
@@ -1584,6 +1670,10 @@ static bool change_pin(ykpiv_state *state, enum enum_action action, const char *
       }
       return false;
 
+    case YKPIV_CONDITION_ERROR:
+        fprintf(stderr, "ERROR: PIN does not meet complexity requirement.\n");
+        return false;
+
     default:
       fprintf(stderr, "Failed changing/unblocking code, error: %s\n", ykpiv_strerror(res));
       return false;
@@ -1773,126 +1863,165 @@ out:
   return ret;
 }
 
-static void print_cert_info(ykpiv_state *state, enum enum_slot slot, const EVP_MD *md,
-    FILE *output) {
-  int object = (int)ykpiv_util_slot_object(get_slot_hex(slot));
-  int slot_name;
-  unsigned char data[YKPIV_OBJ_MAX_SIZE] = {0};
-  unsigned long len = sizeof(data);
-  X509 *x509 = NULL;
-  X509_NAME *subj;
-  BIO *bio = NULL;
-
-  if(ykpiv_fetch_object(state, object, data, &len) != YKPIV_OK) {
-    return;
-  }
-
-  slot_name = get_slot_hex(slot);
-
-  fprintf(output, "Slot %x:\t", slot_name);
-
-  unsigned char certdata[YKPIV_OBJ_MAX_SIZE * 10] = {0};
-  size_t certdata_len = sizeof(certdata);
-  if(ykpiv_util_get_certdata(data, len, certdata, &certdata_len) != YKPIV_OK) {
-    fprintf(output, "Failed to get certificate data\n");
-    return;
-  }
-
-  const unsigned char *certdata_ptr = certdata;
-  x509 = d2i_X509(NULL, &certdata_ptr, certdata_len);
-  if (x509 == NULL) {
-    fprintf(output, "Parse error.\n");
-    return;
-  }
-
-  unsigned int md_len = sizeof(data);
-  const ASN1_TIME *not_before, *not_after;
-
-  EVP_PKEY *key = X509_get_pubkey(x509);
-  if(!key) {
-    fprintf(output, "Parse error.\n");
-    goto cert_out;
-  }
-  fprintf(output, "\n\tAlgorithm:\t");
-  switch(get_algorithm(key)) {
+static void print_algorithm_string(uint8_t algorithm, FILE *output) {
+  switch (algorithm) {
     case YKPIV_ALGO_RSA1024:
-      fprintf(output, "RSA1024\n");
+      fprintf(output, "RSA1024");
       break;
     case YKPIV_ALGO_RSA2048:
-      fprintf(output, "RSA2048\n");
+      fprintf(output, "RSA2048");
       break;
     case YKPIV_ALGO_RSA3072:
-      fprintf(output, "RSA3072\n");
+      fprintf(output, "RSA3072");
       break;
     case YKPIV_ALGO_RSA4096:
-      fprintf(output, "RSA4096\n");
+      fprintf(output, "RSA4096");
       break;
     case YKPIV_ALGO_ECCP256:
-      fprintf(output, "ECCP256\n");
+      fprintf(output, "ECCP256");
       break;
     case YKPIV_ALGO_ECCP384:
-      fprintf(output, "ECCP384\n");
+      fprintf(output, "ECCP384");
       break;
     case YKPIV_ALGO_ED25519:
-      fprintf(output, "ED25519\n");
+      fprintf(output, "ED25519");
       break;
     case YKPIV_ALGO_X25519:
-      fprintf(output, "X25519\n");
+      fprintf(output, "X25519");
       break;
     default:
-      fprintf(output, "Unknown\n");
+      fprintf(output, "Unknown");
   }
-  EVP_PKEY_free(key);
+}
 
-  subj = X509_get_subject_name(x509);
-  if(!subj) {
-    fprintf(output, "Parse error.\n");
-    goto cert_out;
-  }
-  fprintf(output, "\tSubject DN:\t");
-  if(X509_NAME_print_ex_fp(output, subj, 0, XN_FLAG_COMPAT) != 1) {
-    fprintf(output, "Failed to write Subject DN.\n");
-    goto cert_out;
-  }
-  fprintf(output, "\n");
-  subj = X509_get_issuer_name(x509);
-  if(!subj) {
-    fprintf(output, "Parse error.\n");
-    goto cert_out;
-  }
-  fprintf(output, "\tIssuer DN:\t");
-  if(X509_NAME_print_ex_fp(output, subj, 0, XN_FLAG_COMPAT) != 1) {
-    fprintf(output, "Failed to write Issuer DN.\n");
-    goto cert_out;
-  }
-  fprintf(output, "\n");
-  if(X509_digest(x509, md, data, &md_len) != 1) {
-    fprintf(output, "Failed to digest data.\n");
-    goto cert_out;
-  }
-  fprintf(output, "\tFingerprint:\t");
-  dump_data(data, md_len, output, false, format_arg_hex);
+static void print_slot_info(ykpiv_state *state, enum enum_slot slot, const EVP_MD *md, FILE *output) {
+  int slot_name = get_slot_hex(slot);
+  int object = (int) ykpiv_util_slot_object(slot_name);
+  unsigned char data[YKPIV_OBJ_MAX_SIZE] = {0};
+  unsigned long len = sizeof(data);
+  unsigned char metadata[YKPIV_OBJ_MAX_SIZE] = {0};
+  size_t metadata_len = sizeof(metadata);
+  ykpiv_metadata slot_md = {0};
+  X509 *x509 = NULL;
+  EVP_PKEY *key = NULL;
+  X509_NAME *subj;
+  BIO *bio = NULL;
+  bool cert_found = false, metadata_found = false;
 
-  bio = BIO_new_fp(output, BIO_NOCLOSE | BIO_FP_TEXT);
-  not_before = X509_get_notBefore(x509);
-  if(not_before) {
-    fprintf(output, "\tNot Before:\t");
-    if(ASN1_TIME_print(bio, not_before) != 1) {
-      fprintf(output, "Failed to write Not Before time.\n");
+  if(ykpiv_fetch_object(state, object, data, &len) == YKPIV_OK) {
+    unsigned char certdata[YKPIV_OBJ_MAX_SIZE * 10] = {0};
+    size_t certdata_len = sizeof(certdata);
+    if(ykpiv_util_get_certdata(data, len, certdata, &certdata_len) != YKPIV_OK) {
+      fprintf(output, "Failed to get certificate data\n");
+      return;
+    }
+
+    const unsigned char *certdata_ptr = certdata;
+    x509 = d2i_X509(NULL, &certdata_ptr, certdata_len);
+    if (x509 == NULL) {
+      fprintf(output, "Parse error.\n");
+      return;
+    }
+    cert_found = true;
+  }
+
+  if (ykpiv_get_metadata(state, slot_name, metadata, &metadata_len) == YKPIV_OK) {
+    if (ykpiv_util_parse_metadata(metadata, metadata_len, &slot_md) != YKPIV_OK) {
+      fprintf(output, "Failed to parse metadata for slot %x\n", slot_name);
+      return;
+    }
+    metadata_found = true;
+  }
+  if (!cert_found && !metadata_found) {
+    return;
+  }
+
+  fprintf(output, "Slot %x:\n", slot_name);
+
+  fprintf(output, "\tPrivate Key Algorithm:\t");
+  if (metadata_found) {
+    print_algorithm_string(slot_md.algorithm, output);
+    fprintf(output, "\n");
+  } else {
+    fprintf(output, "EMPTY\n");
+  }
+
+  if (cert_found) {
+    unsigned int md_len = sizeof(data);
+    const ASN1_TIME *not_before, *not_after;
+
+    key = X509_get_pubkey(x509);
+    if(!key) {
+      fprintf(output, "Parse error.\n");
+      goto cert_out;
+    }
+    fprintf(output, "\tPublic Key Algorithm:\t");
+    print_algorithm_string(get_algorithm(key), output);
+    fprintf(output, "\n");
+
+    subj = X509_get_subject_name(x509);
+    if(!subj) {
+      fprintf(output, "Parse error.\n");
+      goto cert_out;
+    }
+    fprintf(output, "\tSubject DN:\t\t");
+    if(X509_NAME_print_ex_fp(output, subj, 0, XN_FLAG_COMPAT) != 1) {
+      fprintf(output, "Failed to write Subject DN.\n");
       goto cert_out;
     }
     fprintf(output, "\n");
-  }
-  not_after = X509_get_notAfter(x509);
-  if(not_after) {
-    fprintf(output, "\tNot After:\t");
-    if(ASN1_TIME_print(bio, not_after) != 1) {
-      fprintf(output, "Failed to write Not After time.\n");
+    subj = X509_get_issuer_name(x509);
+    if(!subj) {
+      fprintf(output, "Parse error.\n");
+      goto cert_out;
+    }
+    fprintf(output, "\tIssuer DN:\t\t");
+    if(X509_NAME_print_ex_fp(output, subj, 0, XN_FLAG_COMPAT) != 1) {
+      fprintf(output, "Failed to write Issuer DN.\n");
       goto cert_out;
     }
     fprintf(output, "\n");
+    if(X509_digest(x509, md, data, &md_len) != 1) {
+      fprintf(output, "Failed to digest data.\n");
+      goto cert_out;
+    }
+    fprintf(output, "\tFingerprint:\t\t");
+    dump_data(data, md_len, output, false, format_arg_hex);
+
+    bio = BIO_new_fp(output, BIO_NOCLOSE | BIO_FP_TEXT);
+    not_before = X509_get_notBefore(x509);
+    if(not_before) {
+      fprintf(output, "\tNot Before:\t\t");
+      if(ASN1_TIME_print(bio, not_before) != 1) {
+        fprintf(output, "Failed to write Not Before time.\n");
+        goto cert_out;
+      }
+      fprintf(output, "\n");
+    }
+    not_after = X509_get_notAfter(x509);
+    if(not_after) {
+      fprintf(output, "\tNot After:\t\t");
+      if(ASN1_TIME_print(bio, not_after) != 1) {
+        fprintf(output, "Failed to write Not After time.\n");
+        goto cert_out;
+      }
+      fprintf(output, "\n");
+    }
   }
+
+  if(cert_found && metadata_found) {
+    EVP_PKEY *md_key = EVP_PKEY_new();
+    if (do_create_public_key(slot_md.pubkey, slot_md.pubkey_len, slot_md.algorithm, &md_key) == YKPIV_OK &&
+        EVP_PKEY_cmp(key, md_key) != 1) {
+        fprintf(output, "\tBeware! Slot private key and certificate do not match\n");
+    }
+    EVP_PKEY_free(md_key);
+  }
+
 cert_out:
+  if(key) {
+    EVP_PKEY_free(key);
+  }
   if(x509) {
     X509_free(x509);
   }
@@ -1931,7 +2060,7 @@ static bool status(ykpiv_state *state, enum enum_hash hash,
   if (ykpiv_get_serial(state, &serial) != YKPIV_OK) {
     fprintf(output_file, "No data available\n");
   } else {
-    fprintf(output_file, "%d\n", serial);
+    fprintf(output_file, "%u\n", serial);
   }
 
   fprintf(output_file, "CHUID:\t");
@@ -1949,12 +2078,15 @@ static bool status(ykpiv_state *state, enum enum_hash hash,
     dump_data(buf, len, output_file, false, format_arg_hex);
   }
 
-  if (slot == slot__NULL)
+  fprintf(output_file, "All non-listed slots are empty\n");
+
+  if (slot == slot__NULL) {
     for (i = 0; i < 24; i++) {
-      print_cert_info(state, i, md, output_file);
+      print_slot_info(state, i, md, output_file);
     }
-  else
-    print_cert_info(state, slot, md, output_file);
+  } else {
+    print_slot_info(state, slot, md, output_file);
+  }
 
   {
     int tries;
@@ -2472,6 +2604,7 @@ int main(int argc, char *argv[]) {
       case action_arg_importMINUS_certificate:
       case action_arg_deleteMINUS_certificate:
       case action_arg_readMINUS_certificate:
+      case action_arg_readMINUS_publicMINUS_key:
       case action_arg_testMINUS_signature:
       case action_arg_testMINUS_decipher:
       case action_arg_attest:
@@ -2539,7 +2672,11 @@ int main(int argc, char *argv[]) {
     return EXIT_FAILURE;
   }
 
-  if((rc = ykpiv_connect_ex(state, args_info.reader_arg, args_info.scp11_given)) != YKPIV_OK) {
+  if(args_info.scp11_given) {
+    fprintf(stderr, "WARNING! '--scp11' flag is deprecated. Please use the '--enc' flag instead to establish communication over an encrypted channel\n");
+  }
+
+  if((rc = ykpiv_connect_ex(state, args_info.reader_arg, (args_info.scp11_given || args_info.enc_given))) != YKPIV_OK) {
     fprintf(stderr, "Failed to connect to yubikey: %s.\n", ykpiv_strerror(rc));
     if (rc == YKPIV_PCSC_SERVICE_ERROR) {
       fprintf(stderr, "Try restarting the PCSC subsystem.\n");
@@ -2639,6 +2776,7 @@ int main(int argc, char *argv[]) {
       case action_arg_unblockMINUS_pin:
       case action_arg_selfsignMINUS_certificate:
       case action_arg_readMINUS_certificate:
+      case action_arg_readMINUS_publicMINUS_key:
       case action_arg_status:
       case action_arg_testMINUS_signature:
       case action_arg_testMINUS_decipher:
@@ -2840,6 +2978,11 @@ int main(int argc, char *argv[]) {
           ret = EXIT_FAILURE;
         }
         break;
+      case action_arg_readMINUS_publicMINUS_key:
+        if (get_public_key(state, args_info.slot_arg, args_info.output_arg, args_info.key_format_arg) == false) {
+          ret = EXIT_FAILURE;
+        }
+        break;
       case action_arg_status:
         if(status(state, args_info.hash_arg, args_info.slot_arg, args_info.output_arg) == false) {
           ret = EXIT_FAILURE;
@@ -2881,22 +3024,28 @@ int main(int argc, char *argv[]) {
         }
         break;
       case action_arg_moveMINUS_key: {
-        int from_slot = get_slot_hex(args_info.slot_arg);
-        int to_slot = get_slot_hex((enum enum_slot) args_info.to_slot_arg);
-        if (move_key(state, from_slot, to_slot) == false) {
+        uint8_t from_slot = get_slot_hex(args_info.slot_arg);
+        uint8_t to_slot = get_slot_hex((enum enum_slot) args_info.to_slot_arg);
+        ykpiv_rc res = ykpiv_move_key(state, from_slot & 0xFF,  to_slot & 0xFF);
+        if (res != YKPIV_OK) {
+          fprintf(stderr, "Failed to moved key: %s\n", ykpiv_strerror(res));
           ret = EXIT_FAILURE;
         } else {
           fprintf(stderr, "Successfully moved key.\n");
         }
         break;
       }
-      case action_arg_deleteMINUS_key:
-        if(move_key(state, get_slot_hex(args_info.slot_arg), 0xFF) == false) {
+      case action_arg_deleteMINUS_key: {
+        uint8_t slot = get_slot_hex(args_info.slot_arg);
+        ykpiv_rc res = ykpiv_move_key(state, slot & 0xFF, 0xFF);
+        if (res != YKPIV_OK) {
+          fprintf(stderr, "Failed to delete key: %s\n", ykpiv_strerror(res));
           ret = EXIT_FAILURE;
         } else {
           fprintf(stderr, "Successfully deleted key.\n");
         }
         break;
+      }
       case action__NULL:
       default:
         fprintf(stderr, "Wrong action. %d.\n", action);
